export type RotorComponent = {
	name: M3RotorName | M4RotorName,
	values: string[],
	turnovers: string[],
}

export type ReflectorComponent = {
	name: M3ReflectorName | M4ReflectorName,
	reflector: string[],
}

export type M3RotorName = "I" | "II" | "III" | "IV" | "V" | "VI" | "VII" | "VIII";
export type M3ReflectorName = "A" | "B" | "C";
export type M4RotorName = "Beta" | "Gamma";
export type M4ReflectorName = "B Thin" | "C Thin";

export const m3Rotots: Record<M3RotorName, RotorComponent> = {
	I: {
		name: "I",
		values: [
			"E", "K", "M", "F", "L", "G",
			"D", "Q", "V", "Z", "N", "T",
			"O", "W", "Y", "H", "X", "U",
			"S", "P", "A", "I", "B", "R",
			"C", "J",
		],
		turnovers: ["R"],
	},
	II: {
		name: "II",
		values: [
			"A", "J", "D", "K", "S", "I",
			"R", "U", "X", "B", "L", "H",
			"W", "T", "M", "C", "Q", "G",
			"Z", "N", "P", "Y", "F", "V",
			"O", "E",
		],
		turnovers: ["F"],
	},
	III: {
		name: "III",
		values: [
			"B", "D", "F", "H", "J", "L", 
			"C", "P", "R", "T", "X", "V",
			"Z", "N", "Y", "E", "I", "W",
			"G", "A", "K", "M", "U", "S",
			"Q", "O",
		],
		turnovers: ["W"],
	},
	IV: {
		name: "IV",
		values: [
			"E", "S", "O", "V", "P", "Z",
			"J", "A", "Y", "Q", "U", "I",
			"R", "H", "X", "L", "N", "F",
			"T", "G", "K", "D", "C", "M",
			"W", "B",
		],
		turnovers: ["K"],
	},
	V: {
		name: "V",
		values: [
			"V", "Z", "B", "R", "G", "I",
			"T", "Y", "U", "P", "S", "D",
			"N", "H", "L", "X", "A", "W",
			"M", "J", "Q", "O", "F", "E",
			"C", "K",
		],
		turnovers: ["A"],
	},
	VI: {
		name: "VI",
		values: [
			"J", "P", "G", "V", "O", "U",
			"M", "F", "Y", "Q", "B", "E",
			"N", "H", "Z", "R", "D", "K",
			"A", "S", "X", "L", "I", "C",
			"T", "W",
		],
		turnovers: ["A", "N"],
	},
	VII: {
		name: "VII",
		values: [
			"N", "Z", "J", "H", "G", "R",
			"C", "X", "M", "Y", "S", "W",
			"B", "O", "U", "F", "A", "I",
			"V", "L", "P", "E", "K", "Q",
			"D", "T",
		],
		turnovers: ["A", "N"],
	},
	VIII: {
		name: "VIII",
		values: [
			"F", "K", "Q", "H", "T", "L",
			"X", "O", "C", "B", "J", "S",
			"P", "D", "Z", "R", "A", "M",
			"E", "W", "N", "I", "U", "Y",
			"G", "V",
		],
		turnovers: ["A", "N"],
	},
 };

export const m3Reflectors: Record<M3ReflectorName, ReflectorComponent> = {
	A: {
		name: "A",
		reflector: [
			"E", "J", "M", "Z", "A", "L",
 			"Y", "X", "V", "B", "W", "F",
 			"C", "R", "Q", "U", "O", "N",
 			"T", "S", "P", "I", "K", "H",
 			"G", "D",
		],
	},
	B: {
		name: "B",
		reflector: [
			"Y", "R", "U", "H", "Q", "S",
 			"L", "D", "P", "X", "N", "G",
 			"O", "K", "M", "I", "E", "B",
 			"F", "Z", "C", "W", "V", "J",
 			"A", "T",
		],
	},
	C: {
		name: "C",
		reflector: [
			"F", "V", "P", "J", "I", "A",
 			"O", "Y", "E", "D", "R", "Z",
 			"X", "W", "G", "C", "T", "K",
 			"U", "Q", "S", "B", "N", "M",
 			"H", "L",
		],
	},
};

export const m4Rotors: Record<M4RotorName, RotorComponent> = {
	Beta: {
		name: "Beta",
		values: [
			"L", "E", "Y", "J", "V", "C",
 			"N", "I", "X", "W", "P", "B",
 			"Q", "M", "D", "R", "T", "A",
 			"K", "Z", "G", "F", "U", "H",
 			"O", "S",
		],
		turnovers: [],
	},
	Gamma: {
		name: "Gamma",
		values: [
			"F", "S", "O", "K", "A", "N",
 			"U", "E", "R", "H", "M", "B",
 			"T", "I", "Y", "C", "W", "L",
 			"Q", "P", "Z", "X", "V", "G",
 			"J", "D",
		],
		turnovers: [],
	},
};

export const m4Reflectors: Record<M4ReflectorName, ReflectorComponent> = {
	"B Thin": {
		name: "B Thin",
		reflector: [
			"E", "N", "K", "Q", "A", "U",
 			"Y", "W", "J", "I", "C", "O",
 			"P", "B", "L", "M", "D", "X",
 			"Z", "V", "F", "T", "H", "R",
 			"G", "S",
		],
	},
	"C Thin": {
		name: "C Thin",
		reflector: [
			"R", "D", "O", "B", "J", "N",
 			"T", "K", "V", "E", "H", "M",
 			"L", "F", "C", "W", "Z", "A",
 			"X", "G", "Y", "I", "P", "S",
 			"U", "Q",
		],
	},
}as const;
