import { checkCharacter } from "./validation";

export class Rotor {
	name: string;
	offset: number;
	values: string[];
	turnovers: string[];

	constructor(
		name: string,
		values: string[],
		turnovers: string[],
		offset: number = 0,
	) {
		this.name = name;
		this.offset = offset;
		this.values = values;
		this.turnovers = turnovers;
	}

	rotate(times: number = 1): boolean {
		if (times < 0) {
			throw new Error("offset must be positive");
		}

		if (!Number.isInteger(times)) {
			throw new Error("offset must be an integer");
		}

		this.offset += times;
		this.offset %= 26;

		const char = String.fromCharCode("A".charCodeAt(0) + this.offset);
		const turnover = this.turnovers.includes(char);

		return turnover;
	}

	position(): string {
		return String.fromCharCode("A".charCodeAt(0) + this.offset);
	}

	in(pos: string): string {
		checkCharacter(pos, "position");

		const posNum = pos.charCodeAt(0);
		const input = (posNum - "A".charCodeAt(0) + this.offset) % 26;

		return this.values[input];
	}

	out(pos: string): string {
		checkCharacter(pos, "position");

		let i = this.values.findIndex((char) => char === pos);
		i -= this.offset;
		if (i < 0) i = 26 + i;
		const str = String.fromCharCode("A".charCodeAt(0) + i);

		return str;
	}
}
